import MspJob
from datetime import time, timedelta
from  variables import BASE, CAM, REF 
import iceCape
from gwlogging import logger
from GwErrors import GwLevelError, GwModeError

CRITICAL = 0
LOW = 1
OK = 2
HIGH = 3

#Lower thresholds for levels

#Ignore anything less than this
IGNORE_V = 6.0
CRITICAL_V = 0

LOW_V = 11.5
OK_V = 12.5
HIGH_V = 13.5

thresholds = {HIGH: HIGH_V, OK: OK_V, LOW: LOW_V, CRITICAL: CRITICAL_V}


def setup(gum, level, mode):
    #generic setup for all levels
    if(mode == BASE):
        gum.turn_on_sensor_output(iceCape.PROBE_1 + iceCape.PROBE_2)
        logger.debug("Probe power turned on")

    gum.clear_job_table()

    if mode == CAM:
        gum.add_job(MspJob.J_P_GumstixPcon,
            time(hour=12),
            timedelta(minutes=60),
            None,
            None,
            None)
    else:
        gum.add_job(MspJob.J_P_GumstixPcon,
            time(hour=12),
            timedelta(days=1),
            None,
            None,
            None)
    logger.info("Gumstix job added")

    if mode ==  CAM:
        gum.add_job(MspJob.J_P_GumstixPCON_OFF,
            time(hour=12, minute=30),
            timedelta(minutes=60),
            None,
            None,
            None)
    else:
        gum.add_job(MspJob.J_P_GumstixPCON_OFF,
            time(hour=14),
            timedelta(days=1),
            None,
            None,
            None)
    logger.info("Gumstix OFF job added")

    sensors_list = (iceCape.BATT_V + iceCape.TEMPERATURE +
        iceCape.X + iceCape.Y + iceCape.Z)
    if(mode == REF):
        sensors_list = (sensors_list + iceCape.CAFE + iceCape.SOLAR +
            iceCape.SOLAR_V + iceCape.WIND_V)
    elif(mode == BASE):
        sensors_list = (sensors_list + iceCape.WIND + iceCape.SOLAR +
            iceCape.SOLAR_V)
    elif mode == CAM:
        pass
    else:
        logger.error("NOT SURE HOW I GOT HERE")
        raise GwModeError()

    gum.add_job(MspJob.J_S_SENSSPCON,
        time(hour=0, minute=30),
        timedelta(minutes=30),
        timedelta(seconds=2),
        iceCape.SENSE_EN + iceCape.BATT_EN,
        sensors_list)
    logger.info("Sensors job added")

    # Set up jobs table
    if level == CRITICAL:
        logger.info("Installing critical jobs table")
    elif level == LOW:
        logger.info("Installing low jobs table")
    elif level == OK:
        logger.info("Installing ok jobs table")
        if mode == BASE or mode == REF:
            gum.add_job(MspJob.J_P_PCON,
                time(hour=14),  # 1970-01-01 14:00:00
                timedelta(days=1),
                timedelta(hours=4),
                iceCape.GPS,
                None)

            logger.info("GPS job added")

        if mode == REF:
            gum.add_job(
                MspJob.J_P_PCON,
                time(hour=12),  # 1970-01-01 12:00:00
                timedelta(days=1),
                timedelta(hours=2),
                iceCape.WIFI_1 + iceCape.WIFI_2 + iceCape.WIFI_PA,
                None)
            logger.info("wifi job added")

    elif level == HIGH:
        logger.info("Installing  high jobs table")
    else:
        logger.error("not sure how I got here")
        raise GwLevelError("Unknown Level")
    #Power on 24/7 peripherals
    if level >= CRITICAL:
        logger.info("Powering on critical peripherals")
        if (mode == REF):
            gum.turn_on_sensor_output(iceCape.WEATHER)
            logger.info("Weather station turned on")
    if level >= LOW:
        logger.info("Powering on  low peripherals")
    if level >= OK:
        logger.info("Powering on ok peripherals")
    if level >= HIGH:
        logger.info("Powering on  high peripherals")
        if mode == BASE or mode == REF:
            gum.turn_on_sensor_output(iceCape.GPS)


def teardown(gum, level, mode):
    #generic tear down for all levels

    if level >= CRITICAL:
        logger.info("Powering off critical peripherals")
    if level >= LOW:
        logger.info("Powering off  low peripherals")
        if (mode == REF):
            gum.turn_off_sensor_output(iceCape.WEATHER)
        elif (mode == BASE):
            gum.turn_off_sensor_output(iceCape.GEOPHONE)
        elif mode is CAM:
            pass
        else:
            logger.error("NOT SURE HOW I GOT HERE")
            raise GwModeError()

    if level >= OK:
        logger.info("Powering off OK peripherals")

    if level >= HIGH:
        logger.info("Powering off  high peripherals")
        gum.turn_off_sensor_output(iceCape.GPS)
        if mode == REF:
            gum.turn_off_power_output(iceCape.WIFI_1 + iceCape.WIFI_2
                + iceCape.WIFI_PA)


def reset(gum):
    gum.output_off(65535)
