# gwschedule.py - handles scheduling of jobs and the gumstix

from os import remove
import re

import gwutil
from conversions import conversions
import batvolts
import gwlevels
from variables import BATV_FILE, OVERIDE_FILE
from gwlogging import logger
from GwErrors import GwLevelError
#BOOT_HOUR = 11
#BOOT_MINUTE = 58


def get_state_from_file():
    f = None
    try:
        f = open(BATV_FILE, "r")
        sample = re.compile("^([0-3])$")
        _match = sample.match(f.readline())
        current_state = _match.group(1)
        return int(current_state)
    except Exception, e:
        #if unknown state assume critical
        logger.error("Error occured when reading back from file")
        print e
        return gwlevels.CRITICAL
    finally:
        if f is not None:
            f.close()


def write_state_to_file(new_state):
    f = [new_state]
    gwutil.writeFile(BATV_FILE, f)


def calculate_state(mode):
    conv = conversions(mode)
    average = batvolts.avg(conv)
    if average == 0:
        logger.warn("No valid average available, using current state")
        return get_state_from_file()
    thresholds = gwlevels.thresholds
    for id in reversed(thresholds.keys()):
        if average > thresholds[id]:
            logger.info("New state: %i" % id)
            return id


def change_state(gum, mode, old_state, new_state, force=False):
    #sanity check if the same state and not being forced do nothing
    if((not force) and (old_state == new_state)):
        logger.info("Same state and not being forced so do nothing")
        return
    logger.info("Packing down level %i" % old_state)
    gwlevels.teardown(gum, old_state, mode)
    logger.info("Setting up level %i" % new_state)
    gwlevels.setup(gum, new_state, mode)


def set_local_state(gum, mode, new_state=None):
    previous_state = get_state_from_file()
    if new_state is None:
        new_state = calculate_state(mode)
    #If state == critical it could be the first time a state is being
    #set since a cold start so force it
    change_state(gum, mode, previous_state, new_state,
                   (new_state == gwlevels.CRITICAL))
    write_state_to_file(new_state)
    return new_state


def set_state(gum, mode):
    current_state = get_state_from_file()
    try:
        f = open(OVERIDE_FILE, "r")
        state = f.readline()
        if state is None or state == '':
            logger.warn("No overide state available")
            return
        if int(state[0]) in range(gwlevels.CRITICAL, gwlevels.HIGH + 1):
            new_state = int(state[0])
            if new_state != current_state:
                if new_state < current_state:
                    change_state(gum, mode, new_state, current_state)
                    write_state_to_file(new_state)
                else:
                    logger.info("Overide state %i > local state %i" % (
                                       new_state, current_state))
                    logger.debug("Keeping local state job schedule")
            else:
                logger.info("Overide state %i == local state %i" % (
                               new_state, current_state))
                logger.debug("No changes to be made to job schedule")
        else:
            raise GwLevelError('Unknown state %i' % int(state[0]))
    except Exception, e:
        print e
        logger.error("Failed to set state to %i using local state %i" % (
                       int(state[0]), current_state))
    finally:
        f.close()
        remove(OVERIDE_FILE)
